-------------------------------------------------------------------------------
-- MrTrader 
--
-- Lightweight addon to access profession information of alts.
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

local L = LibStub("AceLocale-3.0"):GetLocale("MrTrader", true);

-- Options for Configuration in the DB
local Options = {
	type = "group",
	get = function(item) return MrTrader.db.profile[item[#item]] end,
	set = function(item, value) MrTrader.db.profile[item[#item]] = value end,
	args = {
		showOppositeFaction = {
			type = "toggle",
			name = L["Show Opposing Faction Alts"],
			desc = L["Will show your alts from the opposing faction in the menu."],
			width = "full",
			order = 1,
		},
		respondToWhispers = {
			type = "toggle",
			name = L["Respond to Whisper Commands"],
			desc = L["Will respond to whispers sent to you asking for skill links."],
			width = "full",
			order = 3,
		},
		whisperAltSkills = {
			type = "toggle",
			name = L["Whisper Skills for Alts"],
			desc = L["Will search your alts for skill links when responding to whisper commands."],
			width = "full",
			order = 3,
		}
	}
}

-- Defaults for options
local Defaults = {
	profile = {
		showOppositeFaction = true,
		respondToWhispers = true,
		whisperAltSkills = false,
		customTradeskillWindow = false,
	},
	realm = {
		skills = {},
	}
}

local ConsoleOptions = {
	type = "group",
	handler = MrTrader,
	args = {
		config = {
			type = "execute",
			name = L["Configure"];
			desc = L["Open configuration dialog"],
			func = function()
				InterfaceOptionsFrame_OpenToCategory(MrTrader.optionFrames.main);
			end
		},
	},
}

local ConsoleCmds = {
	"mrt",
	"mrtrader",
}

-- List of tradeskills
-- Key: Spell ID
-- Value: true if this skill has a tradeskill link you can share
local TradeSkills = {
	[2259] = true, -- Alchemy
	[2018] = true, -- Blacksmithing
	[2550] = true, -- Cooking
	[7411] = true, -- Enchanting
	[4036] = true, -- Engineering
	[3273] = true, -- First Aid
	[45357] = true, -- Inscription
	[25229] = true, -- Jewelcrafting
	[2108] = true, -- Leatherworking
	[53428] = false, -- Runeforging
	[2656] = false, -- Smelting
	[3908] = true, -- Tailoring
}

function MrTrader:Load()
	self.db = LibStub("AceDB-3.0"):New("MrTraderDB", Defaults, "Default");

	-- Convert from v0.1 to v0.3 DBs
	if self.db.factionrealm.skills then
		if self.db.realm.skills then
			self.db.realm.skills = self:TableJoin(self.db.realm.skills, self.db.factionrealm.skills);
		else
			self.db.realm.skills = self.db.factionrealm.skills;
		end

		self.db.factionrealm.skills = nil;

		for name, skillTable in pairs(self.db.realm.skills) do
			skillTable["faction"] = UnitFactionGroup("player");
			self.db.realm.skills[name] = skillTable;
		end
	end

	-- Set a DB version for easier upgrading post v0.3
	if self.db.global.version == nil then
		self.db.global.version = 2;
	end
	
	self:RegisterOptions();
end

function MrTrader:RegisterOptions()
	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	
	local config = LibStub("AceConfig-3.0");
	config:RegisterOptionsTable("MrTrader", ConsoleOptions, ConsoleCmds);
	
	local registry = LibStub("AceConfigRegistry-3.0");
	registry:RegisterOptionsTable("MrTrader Options", Options);
	registry:RegisterOptionsTable("MrTrader Profiles", profiles);
	
	local dialog = LibStub("AceConfigDialog-3.0");
	self.optionFrames = {
		main = dialog:AddToBlizOptions("MrTrader Options", L["MrTrader"]),
		profiles = dialog:AddToBlizOptions("MrTrader Profiles", L["Profiles"], L["MrTrader"]),
	};
end

function MrTrader:QuerySkills()
	local currentPlayer = UnitName("player");
	local currentFaction = UnitFactionGroup("player");	
	local skillTable = {};
	local oldTable = {};
	local didGrabUpdate = true;
	
	if self.db.realm.skills[currentPlayer] then
		oldTable = self.db.realm.skills[currentPlayer];
	end

	for skillID, usesLink in pairs(TradeSkills) do
		skillName = GetSpellInfo(skillID);
		doesExist, tradeLink = GetSpellLink(skillName);
		
		if doesExist and tradeLink then
			skillTable[skillID] = tradeLink;
		elseif doesExist and oldTable[skillID] then
			skillTable[skillID] = oldTable[skillID];
			didGrabUpdate = false;
		elseif doesExist then
			skillTable[skillID] = false;
			if usesLink then
				didGrabUpdate = false;
			end
		end
	end
	
	if self:TableSize(skillTable) > 0 then
		-- Saving the new table, copy over values that we want to keep
		skillTable["ignored"] = oldTable["ignored"];
		skillTable["faction"] = currentFaction;	

		self.db.realm.skills[currentPlayer] = skillTable;
		return didGrabUpdate;
	else
		self.db.realm.skills[currentPlayer] = nil;
		return true;
	end
end

function MrTrader:GetSkillRank(skillLink)
	local _, _, currentRank, maxRank = strsplit(':', skillLink, 4);
	
	return tonumber(currentRank);
end

function MrTrader:MatchPartialCraftSkillName(partialName)
	local matchedSkillID = nil;
	local matchCount = 0;
	
	partialName = partialName:upper();

	for skillID, skillCanCraft in pairs(TradeSkills) do
		if skillCanCraft then
			skillName = GetSpellInfo(skillID);
			skillName = skillName:upper();
			start = string.find( skillName , partialName , 1, true );
			
			if start then 
				matchCount = matchCount + 1;
				matchedSkillID = skillID;
			end
		end
	end
	
	if matchCount > 1 then
		return "ambiguous";
	else
		return matchedSkillID;
	end
end

function MrTrader:GetBestSkillLink(skillID)
	local currentFaction = UnitFactionGroup("player");	
	local currentPlayer = UnitName("player");
	local link = nil;
		
	local currentTable = self.db.realm.skills[currentPlayer];
	
	if self.db.realm.skills[currentPlayer][skillID] ~= nil then
		return self.db.realm.skills[currentPlayer][skillID];
	end
		
	if self.db.profile.whisperAltSkills then
		local currentLink = nil;
		local currentRank = 0;
	
		for playerName, skillList in pairs(self.db.realm.skills) do
			-- Find out if we need to search this character, and if it is in the same faction
			local sameFaction = true;
			if skillList["faction"] and skillList["faction"] ~= currentFaction then
				sameFaction = false;
			end
			
			if sameFaction and skillList[skillID] then
				local skillRank = MrTrader:GetSkillRank(skillList[skillID]);
				if skillRank > currentRank then
					currentLink = skillList[skillID];
					currentRank = skillRank;
				end
			end
		end
		
		return currentLink;
	end
	
	return nil;
end

function MrTrader:GetSkillTable()
	return self.db.realm.skills;
end

function MrTrader:RemoveCharacter(name)
	self.db.realm.skills[name] = nil;
end

function MrTrader:IgnoreCharacter(name)
	self.db.realm.skills[name]["ignored"] = true;
end

function MrTrader:UnignoreCharacter(name)
	self.db.realm.skills[name]["ignored"] = nil;
end

function MrTrader:CharacterIsIgnored(name)
	return self.db.realm.skills[name]["ignored"];
end